package com.ekexiu.project.platform.article;

import com.ekexiu.project.platform.system.vo.SessionManager;
import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.web.annotation.LoginUser;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.util.PageQueryResult;
import org.jfw.util.StringUtil;
import org.jfw.util.auth.AuthUser;
import org.jfw.util.exception.JfwBaseException;
import org.jfw.util.jdbc.JdbcUtil;
import org.jfw.util.jdbc.PreparedStatementConfig;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;

@Path("/article")
public class ArticleService {

    @Autowrie
    private ArticleDao articleDao;

    public ArticleDao getArticleDao() {
        return articleDao;
    }

    public void setArticleDao(ArticleDao articleDao) {
        this.articleDao = articleDao;
    }

    @Path("/publish")
    @Post
    public String publishArticle(@JdbcConn(true) Connection con, Article article, @Nullable String[] professors, @Nullable String[] orgs, @Nullable String[] comps, @LoginUser SessionManager sessionManager) throws SQLException {
        String id = StringUtil.buildUUID();
        article.setId(id);
        article.setPublished(true);
        article.setActived(true);
        article.setCreator(sessionManager.getId());
        article.setModifier(sessionManager.getId());
        relate(con, article, professors, orgs, comps, id);
        articleDao.insert(con, article);
        return id;
    }

    @Path("/draft")
    @Post
    public String draftArticle(@JdbcConn(true) Connection con, Article article, @Nullable String[] professors, @Nullable String[] orgs, @Nullable String[] comps, @LoginUser SessionManager sessionManager) throws SQLException {
        String id = StringUtil.buildUUID();
        article.setId(id);
        article.setPublished(false);
        article.setActived(true);
        article.setCreator(sessionManager.getId());
        article.setModifier(sessionManager.getId());
        relate(con, article, professors, orgs, comps, id);
        articleDao.insert(con, article);
        return id;
    }

    @Path("/draft/save")
    @Post
    public int draftSave(@JdbcConn(true) Connection con,  Article article, @Nullable String[] professors, @Nullable String[] orgs, @Nullable String[] comps, @LoginUser SessionManager sessionManager) throws SQLException, JfwBaseException {
        if (article.getId() != null) {
            article.setModifier(sessionManager.getId());
            articleDao.deleteArticlePro(con, article.getId());
            articleDao.deleteArticleOrg(con, article.getId());
            articleDao.deleteArticleComp(con, article.getId());
            relate(con, article, professors, orgs, comps, article.getId());
            return articleDao.draftUpdate(con, article);
        } else {
            throw new JfwBaseException(-300001, "bad parameter:id");
        }
    }

    @Path("/draft/publish")
    @Post
    public int draftPublish(@JdbcConn(true) Connection con, Article article, @Nullable String[] professors, @Nullable String[] orgs, @Nullable String[] comps, @LoginUser SessionManager sessionManager) throws SQLException, JfwBaseException {
        if (article.getId() != null) {
            article.setPublished(true);
            article.setModifier(sessionManager.getId());
            articleDao.deleteArticlePro(con, article.getId());
            articleDao.deleteArticleOrg(con, article.getId());
            articleDao.deleteArticleComp(con, article.getId());
            relate(con, article, professors, orgs, comps, article.getId());
            return articleDao.draftUpdate(con, article);
        } else {
            throw new JfwBaseException(-300001, "bad parameter:id");
        }
    }

    @LoginUser
    @Path("/pq")
    @Get
    public PageQueryResult<Article> pageQuery(@JdbcConn Connection con, @Nullable String title, @Nullable String catalog, @Nullable Boolean published, int pageSize, int pageNo) throws SQLException {
        return articleDao.pageQuery(con, title == null ? null : "%" + title + "%", catalog, published, pageSize, pageNo);
    }

    @LoginUser
    @Path("/qo")
    @Get
    public Article queryArticle(@JdbcConn Connection con, String id) throws SQLException {
        return articleDao.queryArticle(con, id);
    }

    @Path("/invalid")
    @Get
    public int invalid(@JdbcConn(true) Connection con, final String id, final @LoginUser AuthUser user) throws SQLException {
        if (articleDao.invalid(con, id) != 0) {
            JdbcUtil.execute(con, "INSERT INTO INVALID_ARTICLE (ID,TITLE,CATALOG,CREATOR,CREATE_TIME,INVALID_TIME,INVALID_OPERATOR) "
                    + "SELECT ID,TITLE,CATALOG,CREATOR,CREATE_TIME,TO_CHAR(NOW(),'YYYYMMDDHH24MISS'),? FROM ARTICLE WHERE ID=?", new PreparedStatementConfig() {
                @Override
                public void config(PreparedStatement ps) throws SQLException {
                    ps.setString(1, user.getId());
                    ps.setString(2, id);

                }
            });
            return 1;
        }
        return 0;
    }

    @LoginUser
    @Path("/valid")
    @Get
    public int valid(@JdbcConn(true) Connection con, final String id) throws SQLException {
        if (articleDao.valid(con, id) != 0) {
            articleDao.delete(con, id);
            return 1;
        }
        return 0;
    }

    @LoginUser
    @Path("/invalid/pq")
    @Get
    public PageQueryResult<InvalidArticle> pageQueryInvalid(@JdbcConn Connection con, @Nullable String catalog, int pageSize, int pageNo) throws SQLException {
        return articleDao.pageQuery(con, catalog, pageSize, pageNo);
    }

    @LoginUser
    @Path("/delete/one")
    @Get
    public void deleteOne(@JdbcConn(true) Connection con, String id) throws SQLException {
        articleDao.delete(con, id);
    }

    @LoginUser
    @Path("/delete/all")
    @Get
    public void deleteAll(@JdbcConn(true) Connection con) throws SQLException {
        JdbcUtil.execute(con, "DELETE FROM INVALID_ARTICLE");
    }


    private void relate(Connection con, Article article, String[] professors, String[] orgs, String[] comps, String id) throws SQLException {
        if (professors != null) {
            String pro = join(Arrays.asList(professors), ",");
            article.setProfessor(pro);
            for (String professor : professors) {
                ArticlePro articlePro = new ArticlePro();
                articlePro.setArticleId(id);
                articlePro.setProfessorId(professor);
                articleDao.insert(con, articlePro);
            }
        }
        if (orgs != null) {
            String o = join(Arrays.asList(orgs), ",");
            article.setOrg(o);
            for (String org : orgs) {
                ArticleOrg articleOrg = new ArticleOrg();
                articleOrg.setArticleId(id);
                articleOrg.setOrgId(org);
                articleDao.insert(con, articleOrg);
            }
        }
        if (comps != null) {
            String c = join(Arrays.asList(comps), ",");
            article.setOrg(c);
            for (String comp : comps) {
                ArticleComp articleComp = new ArticleComp();
                articleComp.setArticleId(id);
                articleComp.setCompId(comp);
                articleDao.insert(con, articleComp);
            }
        }
    }

    public static String join(Collection collection, String split) {
        StringBuilder sb = new StringBuilder();
        for (Iterator iterator = collection.iterator(); iterator.hasNext(); sb.append((String) iterator.next())) {
            if (sb.length() != 0) {
                sb.append(split);
            }
        }
        return sb.toString();
    }
}
