package com.ekexiu.project.platform.product;

import com.ekexiu.project.platform.system.vo.SessionManager;
import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.web.annotation.LoginUser;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.operate.Put;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.RequestBody;
import org.jfw.util.PageQueryResult;
import org.jfw.util.StringUtil;
import org.jfw.util.json.JsonService;
import org.jfw.util.reflect.TypeReference;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;

/**
 * Created by TT on 2018/11/26.
 */
@Path("/product")
public class ProductService {

    @Autowrie
    private ProductDao productDao;

    public ProductDao getProductDao() {
        return productDao;
    }

    public void setProductDao(ProductDao productDao) {
        this.productDao = productDao;
    }

    @Path("/insert")
    @Post
    public String insert(@JdbcConn(true) Connection con, ProductInfo product, @LoginUser SessionManager manager) throws SQLException {
        String id = StringUtil.buildUUID();
        product.setId(id);
        product.setCreator(manager.getId());
        product.setModifier(manager.getId());
        product.setActived(true);

        String kw = product.getKeyword();
        if (kw != null) {
            List<String> keywords = JsonService.fromJson(kw, new TypeReference<List<String>>() {
            }.getType());
            if (keywords.size() != 0) {
                int i = 1;
                for (String keyword : keywords) {
                    ProductKeyWord productKeyWord = new ProductKeyWord();
                    productKeyWord.setId(id);
                    productKeyWord.setKeyword(keyword);
                    productKeyWord.setNum(i++);
                    productDao.insert(con, productKeyWord);
                }
            }
        }
        productDao.insert(con, product);
        return id;

    }

    @Path("/update")
    @Put
    public int update(@JdbcConn(true) Connection con, @RequestBody ProductInfo product, @LoginUser SessionManager manager) throws SQLException {
        String id = product.getId();
        product.setModifier(manager.getId());
        product.setActived(true);
        productDao.deleteKeyWord(con, id);
        String kw = product.getKeyword();
        if(kw!=null) {
            List<String> keywords = JsonService.fromJson(kw, new TypeReference<List<String>>() {
            }.getType());
            if (keywords.size() != 0) {
                int i = 1;
                for (String keyword : keywords) {
                    ProductKeyWord productKeyWord = new ProductKeyWord();
                    productKeyWord.setId(id);
                    productKeyWord.setKeyword(keyword);
                    productKeyWord.setNum(i++);
                    productDao.insert(con, productKeyWord);
                }
            }
        }
        return productDao.update(con, product);
    }


    @LoginUser
    @Path("/qo")
    @Get
    public Product query(@JdbcConn Connection con, String id) throws SQLException {
        return productDao.query(con, id);
    }

    @LoginUser
    @Path("/qo/keyword")
    @Get
    public List<ProductKeyWord> queryKeyWord(@JdbcConn Connection con, String id) throws SQLException {
        return productDao.queryKeyWord(con, id);
    }

    @LoginUser
    @Path("/pq")
    @Get
    public PageQueryResult<Product> pageQuery(@JdbcConn Connection con, String companyId, int pageSize, int pageNo) throws SQLException {
        return productDao.pageQuery(con, companyId, pageSize, pageNo);
    }

    @Path("/delete")
    @Post
    public int logicDelete(@JdbcConn(true) Connection con, String id, @LoginUser SessionManager manager) throws SQLException {
        return productDao.delete(con, id, manager.getId());
    }

}
