package com.ekexiu.project.platform.system.dao;

import com.ekexiu.project.platform.system.po.InvalidUser;
import com.ekexiu.project.platform.system.po.Manager;
import com.ekexiu.project.platform.system.po.SysRight;
import com.ekexiu.project.platform.system.po.User;
import com.ekexiu.project.platform.system.po.UserLoginLog;
import com.ekexiu.project.platform.system.vo.ManagerInfo;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.orm.annotation.dao.Batch;
import org.jfw.apt.orm.annotation.dao.Column;
import org.jfw.apt.orm.annotation.dao.DAO;
import org.jfw.apt.orm.annotation.dao.method.Exclude;
import org.jfw.apt.orm.annotation.dao.method.From;
import org.jfw.apt.orm.annotation.dao.method.IncludeFixSet;
import org.jfw.apt.orm.annotation.dao.method.OrderBy;
import org.jfw.apt.orm.annotation.dao.method.SetSentence;
import org.jfw.apt.orm.annotation.dao.method.Where;
import org.jfw.apt.orm.annotation.dao.method.operator.DeleteWith;
import org.jfw.apt.orm.annotation.dao.method.operator.Insert;
import org.jfw.apt.orm.annotation.dao.method.operator.PageQuery;
import org.jfw.apt.orm.annotation.dao.method.operator.PageSelect;
import org.jfw.apt.orm.annotation.dao.method.operator.QueryOne;
import org.jfw.apt.orm.annotation.dao.method.operator.QueryValList;
import org.jfw.apt.orm.annotation.dao.method.operator.SelectOne;
import org.jfw.apt.orm.annotation.dao.method.operator.Update;
import org.jfw.apt.orm.annotation.dao.method.operator.UpdateWith;
import org.jfw.apt.orm.annotation.dao.param.Alias;
import org.jfw.apt.orm.annotation.dao.param.GroupSqlColumn;
import org.jfw.apt.orm.annotation.dao.param.Set;
import org.jfw.apt.orm.annotation.dao.param.UnEquals;
import org.jfw.apt.orm.core.defaultImpl.StringHandler;
import org.jfw.util.PageQueryResult;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;


@DAO
public interface UserDao {

    @Nullable
    @SelectOne
    User query(Connection con, String id) throws SQLException;

    @Nullable
    @SelectOne
    User queryByLoginPhone(Connection con, String loginPhone, Boolean actived) throws SQLException;

    @Nullable
    @SelectOne
    User login(Connection con, String account, String passwd) throws SQLException;

    @Insert
    int insert(Connection con, User user) throws SQLException;

    @Insert
    int insert(Connection con, UserLoginLog uli) throws SQLException;

    @UpdateWith
    @From(UserLoginLog.class)
    int updateLoginLog(Connection con, String id, @Set String lastLoginTime) throws SQLException;

    @Update
    @Exclude({"passwd", "actived", "loginPhone"})
    int update(Connection con, User user) throws SQLException;

    @UpdateWith
    @From(Manager.class)
    int changePasswd(Connection con, String id, @Set @Alias("passwd") String newPasswd) throws SQLException;

    @UpdateWith
    @From(User.class)
    int resetPasswd(Connection con, String loginPhone, @Set String passwd) throws SQLException;

    @UpdateWith
    @From(User.class)
    @Where("ACTIVED = '1'")
    @SetSentence("ACTIVED = '0'")
    int invalid(Connection con, String id) throws SQLException;

    @UpdateWith
    @From(User.class)
    @Where("ACTIVED = '0'")
    @SetSentence("ACTIVED = '1'")
    int valid(Connection con, String id) throws SQLException;

    @Insert
    int invalidUser(Connection con, InvalidUser invalidUser) throws SQLException;

    @DeleteWith
    @From(InvalidUser.class)
    int validUser(Connection con, String id) throws SQLException;

    @PageSelect
    @Where("ACTIVED ='1'")
    @OrderBy("ORDER BY CREATE_TIME DESC")
    PageQueryResult<User> pageQuery(Connection con, @GroupSqlColumn(handlerClass = StringHandler.class, value = {"ACCOUNT LIKE ?", "LOGIN_PHONE LIKE ?"}, isAnd = false) @Nullable String key, int pageSize, int pageNo) throws SQLException;

    @PageSelect
    @OrderBy("ORDER BY INVALID_TIME DESC")
    PageQueryResult<InvalidUser> pageQueryInvalidUser(Connection con, @GroupSqlColumn(handlerClass = StringHandler.class, value = {"ACCOUNT LIKE ?", "LOGIN_PHONE LIKE ?"}, isAnd = false) @Nullable String key, int pageSize, int pageNo) throws SQLException;

    @Nullable
    @SelectOne
    InvalidUser queryInvalidUser(Connection con, String id) throws SQLException;

    /**
     * 后台用户
     */
    @Insert
    int insert(Connection con, Manager user) throws SQLException;

    @Insert
    @Batch
    int[] insert(Connection con, SysRight[] rightcodes) throws SQLException;

    @DeleteWith
    @From(SysRight.class)
    int delete(Connection con, String uid, @UnEquals String rightCode) throws SQLException;


    @Update
    @Where("ACTIVED ='1'")
    @IncludeFixSet
    @Exclude(value = {"invalidTime", "invalidOperator", "passwd", "actived"})
    int update(Connection con, Manager user) throws SQLException;

    @UpdateWith
    @From(Manager.class)
    @Where("ACTIVED ='1'")
    @SetSentence("ACTIVED ='0',INVALID_TIME=TO_CHAR(NOW(),'YYYYMMDDHH24MISS')")
    int disable(Connection con, String id, @Set String invalidOperator) throws SQLException;

    @UpdateWith
    @From(Manager.class)
    @Where("ACTIVED ='0'")
    @SetSentence("ACTIVED ='1'")
    int enable(Connection con, String id) throws SQLException;

    @Nullable
    @QueryOne
    @From(Manager.class)
    ManagerInfo managerLogin(Connection con, String account, String passwd) throws SQLException;

    @Nullable
    @QueryValList
    @From(SysRight.class)
    @Column(value = "RIGHT_CODE", handlerClass = StringHandler.class)
    List<String> right(Connection con, String uid) throws SQLException;

    @Nullable
    @SelectOne
    Manager queryManagerByLoginPhone(Connection con, String account) throws SQLException;

    @UpdateWith
    @From(Manager.class)
    int changeManagerPasswd(Connection con, String id, @Set String passwd) throws SQLException;

    @UpdateWith
    @From(Manager.class)
    int managerResetPasswd(Connection con, String id, @Set String passwd) throws SQLException;

    @PageQuery
    @From(Manager.class)
    @OrderBy("ORDER BY CREATE_TIME DESC")
    PageQueryResult<ManagerInfo> managerPageQuery(Connection con, @UnEquals String id, boolean actived, int pageSize, int pageNo) throws SQLException;

    @Nullable
    @QueryOne
    @From(Manager.class)
    ManagerInfo managerQuery(Connection con, String id) throws SQLException;
}
