package com.ekexiu.project.platform.system.service;

import com.ekexiu.project.platform.system.dao.UserDao;
import com.ekexiu.project.platform.system.po.InvalidUser;
import com.ekexiu.project.platform.system.po.Manager;
import com.ekexiu.project.platform.system.po.SysRight;
import com.ekexiu.project.platform.system.po.User;
import com.ekexiu.project.platform.system.vo.ManagerInfo;
import com.ekexiu.project.platform.system.vo.SessionManager;
import com.ekexiu.project.platform.system.vo.SessionUser;
import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.web.annotation.LoginUser;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.method.InvalidSession;
import org.jfw.apt.web.annotation.method.SetSession;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.operate.Put;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.PathVar;
import org.jfw.apt.web.annotation.param.RequestBody;
import org.jfw.apt.web.annotation.param.RequestParam;
import org.jfw.util.PageQueryResult;
import org.jfw.util.StringUtil;
import org.jfw.util.auth.AuthUser;
import org.jfw.util.exception.JfwBaseException;
import org.jfw.util.jdbc.JdbcUtil;
import org.jfw.util.jdbc.PreparedStatementConfig;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Path("/sys")
public class SysService {

    private String fixRightCode = "0";

    @Autowrie
    private UserDao userDao;

    public UserDao getUserDao() {
        return userDao;
    }

    public void setUserDao(UserDao userDao) {
        this.userDao = userDao;
    }

    @SetSession("JFW_SESSION_LOGIN_USER=result")
    @Path("/login")
    @Post
    public SessionManager managerLogin(@JdbcConn Connection con, String account, String pw) throws SQLException, JfwBaseException {
        ManagerInfo user = userDao.managerLogin(con, account, StringUtil.md5(pw));
        if (user != null) {
            if (!user.isActived()) {
                throw new JfwBaseException(-200001, "user is disabled");

            }
            List<String> rightList = userDao.right(con, user.getId());
            user.setRightCode(rightList.toArray(new String[0]));
            return makeSessionManager(user);
        }
        return null;
    }

    @LoginUser
    @Get
    @Path("/exists")
    public boolean existsManager(@JdbcConn Connection con, String account) throws SQLException {
        return null != userDao.queryManagerByLoginPhone(con, account);
    }

    @Get
    @Path("/logout")
    @InvalidSession
    public void managerLogout() {
    }


    @Get
    @Path("/manager")
    public SessionManager get(@LoginUser(false) SessionManager user) {
        return user;
    }

    @Post
    @Path("/manager/changepasswd")
    public int changeManagerPasswd(@JdbcConn(true) Connection con, @LoginUser SessionManager loginUser, String npw) throws SQLException {
        return userDao.changeManagerPasswd(con, loginUser.getId(), StringUtil.md5(npw));
    }

    @Put
    @Path("/manager")
    public int modifyManager(@JdbcConn(true) Connection con, @LoginUser SessionManager loginUser,
                             @RequestBody Manager user) throws SQLException {
        user.setModifier(loginUser.getId());
        user.setId(loginUser.getId());
        return userDao.update(con, user);
    }

    @Get
    @Path("/disable")
    public void managerDisable(@JdbcConn(true) Connection con, String id, @LoginUser SessionManager loginUser) throws SQLException {
        userDao.disable(con, id, loginUser.getId());
    }

    @LoginUser
    @Get
    @Path("/enable")
    public void managerEnable(@JdbcConn(true) Connection con, String id) throws SQLException {
        userDao.enable(con, id);
    }

    @LoginUser
    @Get
    @Path("/resetpw")
    public void managerResetPw(@JdbcConn(true) Connection con, String id) throws SQLException {
        userDao.managerResetPasswd(con, id, StringUtil.md5("123456"));
    }

    @Get
    @Path("/pq")
    public PageQueryResult<SessionManager> managerPageQuery(@JdbcConn Connection con, @LoginUser SessionManager user, boolean actived, int pageSize, int pageNo) throws SQLException {
        PageQueryResult<ManagerInfo> pageQueryResult = userDao.managerPageQuery(con, user.getId(), actived, pageSize, pageNo);
        List<ManagerInfo> managers = pageQueryResult.getData();
        List<SessionManager> sessionManagers = new ArrayList<>();
        if (!managers.isEmpty()) {
            for (ManagerInfo manager : managers) {
                sessionManagers.add(makeSessionManager(manager));
            }
            PageQueryResult<SessionManager> ret = new PageQueryResult<>();
            ret.setPageNo(pageQueryResult.getPageNo());
            ret.setPageSize(pageQueryResult.getPageSize());
            ret.setTotal(pageQueryResult.getTotal());
            ret.setData(sessionManagers);
            return ret;
        }
        return null;
    }

    @LoginUser
    @Get
    @Path("/qo")
    public SessionManager managerQueryOne(@JdbcConn Connection con, String id) throws SQLException {
        ManagerInfo managerInfo = userDao.managerQuery(con, id);
        List<String> rightList = userDao.right(con, managerInfo.getId());
        managerInfo.setRightCode(rightList.toArray(new String[0]));
        return makeSessionManager(managerInfo);
    }

    @Path("/new")
    @Post
    public String addUser(@JdbcConn(true) Connection con, @LoginUser SessionManager au, @RequestParam(excludeFields = {"id", "acitved", "createor"}) Manager user, String[] rightCode) throws SQLException {
        String id = StringUtil.buildUUID();
        user.setActived(true);
        user.setCreator(au.getId());
        user.setModifier(au.getId());
        user.setId(id);
        user.setInvalidOperator(null);
        user.setInvalidTime(null);
        user.setPasswd(StringUtil.md5("123456"));
        userDao.insert(con, user);
        if (rightCode != null && rightCode.length > 0) {
            SysRight rights[] = new SysRight[rightCode.length];
            for (int i = 0; i < rightCode.length; ++i) {
                rights[i] = new SysRight();
                rights[i].setUid(id);
                rights[i].setRightCode(rightCode[i]);
            }
            userDao.insert(con, rights);
        }
        return id;
    }

    @Path("/renew")
    @Put
    public void renewUser(@JdbcConn(true) Connection con, @LoginUser SessionManager au, @RequestBody ManagerInfo user) throws SQLException {
        user.setModifier(au.getId());
        userDao.update(con, user);
        userDao.delete(con, user.getId(), this.fixRightCode);
        String[] rightCode = user.getRightCode();
        if (rightCode != null && rightCode.length > 0) {
            SysRight rights[] = new SysRight[rightCode.length];
            for (int i = 0; i < rightCode.length; ++i) {
                rights[i] = new SysRight();
                rights[i].setUid(user.getId());
                rights[i].setRightCode(rightCode[i]);
            }
            userDao.insert(con, rights);
        }
    }

    @Path("/right/{uid}")
    @Put
    public void rightUser(@JdbcConn(true) Connection con, @LoginUser AuthUser au, @PathVar String uid, @RequestBody String[] rightCode) throws SQLException {
        userDao.delete(con, uid, this.fixRightCode);
        if (rightCode != null && rightCode.length > 0) {
            SysRight rights[] = new SysRight[rightCode.length];
            for (int i = 0; i < rightCode.length; ++i) {
                rights[i] = new SysRight();
                rights[i].setUid(uid);
                rights[i].setRightCode(rightCode[i]);
            }
            userDao.insert(con, rights);
        }
    }

    @Path("/user/invalid")
    @Post
    public int invalidUser(@JdbcConn(true) Connection con, @LoginUser SessionManager manager, String id, String reason) throws SQLException {
        if (userDao.invalid(con, id) != 0) {
            User user = userDao.query(con, id);
            InvalidUser invalidUser = new InvalidUser();
            invalidUser.setName(user.getName());
            invalidUser.setId(user.getId());
            invalidUser.setAccount(user.getAccount());
            invalidUser.setAddr(user.getAddr());
            invalidUser.setComp(user.getComp());
            invalidUser.setEmail(user.getEmail());
            invalidUser.setHead(user.getHead());
            invalidUser.setJob(user.getJob());
            invalidUser.setLinkPhone(user.getLinkPhone());
            invalidUser.setLoginPhone(user.getLoginPhone());
            invalidUser.setInvalidOperator(manager.getId());
            invalidUser.setInvalidReason(reason);
            invalidUser.setSex(user.getSex());
            userDao.invalidUser(con, invalidUser);
            return 1;
        }
        return 0;
    }

    @LoginUser
    @Path("/user/valid")
    @Get
    public void validUser(@JdbcConn(true) Connection con, String id) throws SQLException {
        userDao.valid(con, id);
        userDao.validUser(con, id);
    }

    @LoginUser
    @Path("/user/pq")
    @Get
    public PageQueryResult<SessionUser> pageQuery(@JdbcConn Connection con, @Nullable String key, int pageSize, int pageNo) throws SQLException {
        PageQueryResult<User> pageQueryResult = userDao.pageQuery(con, key == null ? null : "%" + key + "%", pageSize, pageNo);
        List<User> users = pageQueryResult.getData();
        List<SessionUser> sessionUsers = new ArrayList<>();
        if (!users.isEmpty()) {
            for (User user : users) {
                sessionUsers.add(makeSessionUser(user));
            }
            PageQueryResult<SessionUser> ret = new PageQueryResult<>();
            ret.setPageNo(pageQueryResult.getPageNo());
            ret.setPageSize(pageQueryResult.getPageSize());
            ret.setTotal(pageQueryResult.getTotal());
            ret.setData(sessionUsers);
            return ret;
        }
        return null;
    }

    @LoginUser
    @Path("/user/invalid/pq")
    @Get
    public PageQueryResult<InvalidUser> pageQueryInvalidUser(@JdbcConn Connection con, @Nullable String key, int pageSize, int pageNo) throws SQLException {
        return userDao.pageQueryInvalidUser(con, key == null ? null : "%" + key + "%", pageSize, pageNo);
    }

    @LoginUser
    @Path("/user/qo")
    @Get
    public SessionUser query(@JdbcConn Connection con, String id) throws SQLException {
        User user = userDao.query(con, id);
        return makeSessionUser(user);
    }

    @LoginUser
    @Path("/user/invalid/qo")
    @Get
    public InvalidUser queryInvalidUser(@JdbcConn Connection con, String id) throws SQLException {
        return userDao.queryInvalidUser(con, id);
    }

    @LoginUser
    @Path("/user/lastTime")
    @Get
    public List<Map<String, Object>> queryLastTime(@JdbcConn Connection con, final String[] ids) throws SQLException {
        final StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append("SELECT ID,LAST_LOGIN_TIME WHERE ID IN(");
        for (String id : ids) {
            stringBuilder.append("?,");
        }
        stringBuilder.setCharAt(stringBuilder.length() - 1, ')');
        return JdbcUtil.queryMaps(con, stringBuilder.toString(), new PreparedStatementConfig() {
            @Override
            public void config(PreparedStatement preparedStatement) throws SQLException {
                int i = 1;
                for (String id : ids) {
                    preparedStatement.setString(i++, id);
                }
            }
        });
    }

    private static SessionManager makeSessionManager(ManagerInfo manager) {
        SessionManager sessionManager = new SessionManager();
        sessionManager.setId(manager.getId());
        sessionManager.setName(manager.getName());
        sessionManager.setAccount(manager.getAccount());
        sessionManager.setPhone(manager.getPhone());
        sessionManager.setEmail(manager.getEmail());
        sessionManager.setJob(manager.getJob());
        sessionManager.setDep(manager.getDep());
        sessionManager.setComp(manager.getComp());
        sessionManager.setActived(manager.isActived());
        sessionManager.setCreateTime(manager.getCreateTime());
        sessionManager.setInvalidTime(manager.getInvalidTime());
        sessionManager.setRightCode(manager.getRightCode());
        return sessionManager;
    }

    private static SessionUser makeSessionUser(User user) {
        SessionUser sessionUser = new SessionUser();
        sessionUser.setAccount(user.getAccount());
        sessionUser.setComp(user.getComp());
        sessionUser.setEmail(user.getEmail());
        sessionUser.setId(user.getId());
        sessionUser.setJob(user.getJob());
        sessionUser.setName(user.getName());
        sessionUser.setActived(user.isActived());
        sessionUser.setAddr(user.getAddr());
        sessionUser.setCreateTime(user.getCreateTime());
        sessionUser.setHead(user.getHead());
        sessionUser.setLinkPhone(user.getLinkPhone());
        sessionUser.setLoginPhone(user.getLoginPhone());
        sessionUser.setModifyTime(user.getModifyTime());
        sessionUser.setSex(user.getSex());
        return sessionUser;
    }


}
